"""\
This file is an installation script for CMFContentPanels (ZPT skins).  It's meant
to be used as an External Method.  To use, add an external method to the
root of the CMF Site that you want ZPT skins registered in with the
configuration:

 id: install_forum
 title: Install ContentPanels Product *optional*
 module name: CMFContentPanels.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Test' tab.  The install function will execute and give
information about the steps it took to register and install the
ZPT skins into the CMF Site instance. 
"""

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod import ExternalMethod

from Acquisition import aq_base
from cStringIO import StringIO
import string
import string 

from Acquisition import Implicit
import Persistence

from Products.CMFContentPanels import ContentPanels_globals, ContentPanels

__version__='0.5'

def install_SubSkin(self, outStream, skinFolder):
    """ Installs a subskin, should be just 1 folder.
    """
    skinstool=getToolByName(self, 'portal_skins')
    for skin in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skin)
        path = map( string.strip, string.split( path,',' ) )
        if not skinFolder in path:
            try:
                path.insert( path.index( 'custom')+1, skinFolder )
            except ValueError:
                path.append(skinFolder)
            path = string.join( path, ', ' )
            skinstool.addSkinSelection( skin, path )
            outStream.write('CMFContentPanels subskin successfully installed into %s.\n' % skin)    
        else:
            outStream.write('CMFContentPanels subskin was already installed into %s.\n' % skin)    

def install_Skins(self, out):
    """
        Add a new skin, 'Plone', copying 'ZPT', if it exists, and then
        add our directories only to it.
    """

    skinstool = getToolByName(self, 'portal_skins')

    try:
        install_SubSkin(self, out, 'contentpanels')
        install_SubSkin(self, out, 'contentpanels/viewlets')
        install_SubSkin(self, out, 'contentpanels/box_skins')
        out.write('CMFContentPanels subskin successfully installed.\n')
    except:
        out.write('CMFContentPanels subskin failed to install.\n')

    try:  
        addDirectoryViews( skinstool, 'skins', ContentPanels_globals )
        out.write( "Added CMFContentPanels directory views to portal_skins.\n" )
    except:
        out.write( 'Unable to add CMFContentpanels directory view to portal_skins.\n')

def install_Actions(self, out):
        # Set actions
        portal_actions = getToolByName(self, 'portal_actions')
        # latest_updates_viewlet / news_viewlet
        viewlet = 'latest_updates_viewlet'
        if viewlet in [action.id for action in portal_actions.listActions()]:
            listA = portal_actions.listActions()
            selections = tuple([i for i in range(0,len(listA)) if listA[i].id==viewlet])
            portal_actions.deleteActions(selections)
        portal_actions.addAction(
            id=viewlet,
            name='Lastest Updates',
            action='string:here/folder_viewlet_macros/macros/latest_updates',
            condition='python: object.isPrincipiaFolderish',
            permission='View',
            category='panel_viewlets',
            visible=1)

        viewlet = 'default_viewlet'
        if viewlet in [action.id for action in portal_actions.listActions()]:
            listA = portal_actions.listActions()
            selections = tuple([i for i in range(0,len(listA)) if listA[i].id==viewlet])
            portal_actions.deleteActions(selections)
        portal_actions.addAction(
            id=viewlet,
            name='Default Viewlet',
            action='string:here/contentpanels_default_viewlet',
            condition='',
            permission='View',
            category='panel_viewlets',
            visible=1)


        portal_types = getToolByName(self, 'portal_types')
        # Document
        portalType = portal_types.Document
        viewlet = 'document_viewlet'
        if viewlet in [action['id'] for action in portalType._actions]:
            listA = portalType._actions
            selections = tuple([i for i in range(0,len(listA)) if listA[i]['id']==viewlet])
            portalType.deleteActions(selections)
        portalType.addAction(
            id=viewlet,
            name='Document Body',
            action='here/document_panel_view',
            permission='View',
            category='panel_viewlets',
            visible=1)

        # Topic
        portalType = portal_types.Topic
        viewlet = 'view_viewlet'
        if viewlet in [action['id'] for action in portalType._actions]:
            listA = portalType._actions
            selections = tuple([i for i in range(0,len(listA)) if listA[i]['id']==viewlet])
            portalType.deleteActions(selections)
        portalType.addAction(
            id=viewlet,
            name='View',
            action='here/topic_panel_view',
            permission='View',
            category='panel_viewlets',
            visible=1)

        # Image
        portalType = portal_types.Image
        viewlet = 'view_viewlet'
        if viewlet in [action['id'] for action in portalType._actions]:
            listA = portalType._actions
            selections = tuple([i for i in range(0,len(listA)) if listA[i]['id']==viewlet])
            portalType.deleteActions(selections)
        portalType.addAction(
            id=viewlet,
            name='View',
            action='here/image_viewlet_macros/macros/view',
            permission='View',
            category='panel_viewlets',
            visible=1)

        out.write("Actions Setup Done\n")

def install(self):
    """ Register the ContentPanels Skins with portal_skins and friends """
    skinstool = getToolByName(self, 'portal_skins')
    wf_tool = getToolByName(self, 'portal_workflow')

    out = StringIO()
    out.write( 'CMFContentPanels installation tool v' + str(__version__) + '.\n')

    install_Skins(self, out)

    install_Actions(self, out)

    # Register with the typestool manually instead of with manage_addTypeInformation
    # as the classes were not registered with utils.ContentInit in __init__.py
    types_tool = getToolByName(self, 'portal_types')
    for t in ContentPanels.factory_type_information:
        if t['id'] not in types_tool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), t)
            types_tool._setObject(t['id'], cfm)
            out.write('Registered %s with the types tool\n' % t['id'])
        else:
            out.write('Skipping "%s" - already in types tool\n' % t['id'])


    self.portal_form.setValidators('contentpanels_edit_property_form', ['validate_document_edit'])

    self.portal_navigation.addTransitionFor('default', 'contentpanels_edit_property_form', 'success', 'script:contentpanels_edit_property')
    self.portal_navigation.addTransitionFor('default', 'contentpanels_edit_property_form', 'failure', 'contentpanels_edit_property_form')
    self.portal_navigation.addTransitionFor('default', 'contentpanels_edit_property', 'success', 'action:view')
    self.portal_navigation.addTransitionFor('default', 'contentpanels_edit_property', 'failure', 'contentpanels_edit_property_form')


    #wf_tool.setChainForPortalTypes( ('ContentPanels',), [])
    #out.write('Removed workflow for ContentPanels')
            
    return out.getvalue()
